<?php

/**
 * @file
 * Configuration pages for CRM Core Profile.
 */

define('DEFAULT_WEIGHT_DELTA', 50);

/**
 * The main screen for CRM Core Profile.
 *
 * This function provides the main screen for managing, editing, cloning, etc.
 */
function crm_core_profile_main() {

  drupal_set_title(t('CRM Core Profile'));
  $output = '';
  $output .= '<h1>' . t('Available profiles') . '</h1>';

  $rows = array();
  foreach (crm_core_profile_load_all() as $profile) {

    // @todo How can we define that there is embedded usage?
    $displays = array();
    if ($profile->page) {
      $displays[] = t('Page');
    }
    if ($profile->block) {
      $displays[] = t('Block');
    }

    $path = '---';
    if ($profile->page) {
      $prepopulate = isset($profile->entities['crm_core_contact']['prepopulate']) ? $profile->entities['crm_core_contact']['prepopulate'] : FALSE;
      if ($prepopulate || $profile->page) {
        $path = l(check_plain($profile->page_path), $profile->page_path);
      }
      else {
        $path = l(check_plain('crm-core/profile/' . $profile->name), 'crm-core/profile/' . $profile->name);
      }
    }

    $status_toggle_path = 'admin/structure/crm-core/profile/manage/' . $profile->name . '/status';
    $status = l($profile->status ? t('Enabled') : t('Disabled'), $status_toggle_path);

    $operations = theme('links', array(
        'links' => array(
          'edit' => array(
            'title' => t('Edit'),
            'href' => 'admin/structure/crm-core/profile/manage/' . $profile->name . '/edit',
          ),
          'settings' => array(
            'title' => t('Settings'),
            'href' => 'admin/structure/crm-core/profile/manage/' . $profile->name . '/settings',
          ),
          'clone' => array(
            'title' => t('Clone'),
            'href' => 'admin/structure/crm-core/profile/manage/' . $profile->name . '/clone',
          ),
          'export' => array(
            'title' => t('Export'),
            'href' => 'admin/structure/crm-core/profile/manage/' . $profile->name . '/export',
          ),
        ),
        'attributes' => array('class' => array('links', 'inline')),
      )
    );

    $rows[] = array(
      'data' => array(
        l(check_plain($profile->label), 'admin/structure/crm-core/profile/manage/' . $profile->name . '/settings'),
        $profile->name,
        implode(', ', $displays),
        $path,
        $status,
        $operations,
      ),
    );
  }

  $header = array(
    t('Name'),
    t('Machine Name'),
    t('Display'),
    t('Path'),
    t('Status'),
    t('Operations'),
  );
  $output .= theme('table', array(
    'header' => $header,
    'rows' => $rows,
    'attributes' => array('id' => 'crm-core-profiles-list'),
    'empty' => t('There are no profiles currently configured.'),
  ));

  return $output;
}

/**
 * Generates a new profile form or edit an existing form.
 */
function crm_core_profile_new_form($form, &$form_state, $profile, $clone = FALSE) {

  $breadcrumb = array(
    l(t('Home'), '<front>'),
    l(t('Administration'), 'admin'),
    l(t('Structure'), 'admin/structure'),
    l(t('CRM Core'), 'admin/structure/crm-core'),
    l(t('CRM Core Profile'), 'admin/structure/crm-core/profile'),
  );
  drupal_set_breadcrumb($breadcrumb);

  global $user;

  if (empty($profile->name)) {
    $form_state['is_new'] = TRUE;
  }

  // Updating profile fields from $form_state
  $profile_fields = array('label', 'name', 'status', 'entities', 'fields');
  if (isset($form_state['values'])) {
    foreach ($profile_fields as $key) {
      if ($form_state['values'][$key]) {
        $profile->$key = $form_state['values'][$key];
      }
    }
  }

  if (empty($form_state['is_new'])) {
    $form['#profile'] = $profile;
  }
  $weight = -DEFAULT_WEIGHT_DELTA;
  $form['#weight'] =& $weight;

  $form['is_new'] = array(
    '#type' => 'value',
    '#value' => 0,
  );

  $form['uid'] = array(
    '#type' => 'value',
    '#value' => $user->uid,
  );

  if ($clone) {
    drupal_set_title(t('Save copy of @profile', array('@profile' => $profile->label)));
    $profile->clone = $clone;
    $profile->label = 'Copy of ' . $profile->label;
    $profile->name = 'copy_of_' . $profile->name;
    $form['is_new']['#value'] = 1;
    $form['created'] = array(
      '#type' => 'value',
      '#value' => time(),
    );
  }
  else {
    // Setting page title.
    if (!empty($form_state['is_new'])) {
      drupal_set_title(t('New profile settings'));
      $form['is_new']['#value'] = 1;
      $form['created'] = array(
        '#type' => 'value',
        '#value' => time(),
      );
    }
    else {
      drupal_set_title(t('Profile settings for @profile', array('@profile' => $profile->label)));
    }
  }

  $form['label'] = array(
    '#type' => 'textfield',
    '#title' => 'Name',
    '#description' => 'Please enter the profile form name',
    '#maxlength' => 28,
    '#required' => TRUE,
    '#default_value' => empty($profile->label) ? '' : $profile->label,
  );

  $form['name'] = array(
    '#type' => 'machine_name',
    '#maxlength' => 28,
    '#machine_name' => array(
      'exists' => 'crm_core_profile_new_form_machine_name_check',
      'source' => array('label'),
    ),
    '#default_value' => empty($profile->name) ? '' : $profile->name,
    '#disabled' => empty($profile->name) || $clone ? FALSE : TRUE,
  );

  $form['status'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enabled'),
    '#description' => t('Check this box to enable the profile. Disabling a profile makes it unavailable for use.'),
    '#default_value' => empty($profile->status) ? FALSE : $profile->status,
  );

  $form['entities'] = array(
    '#type' => 'vertical_tabs',
    '#tree' => TRUE,
  );

  $entity_handlers = module_invoke_all('crm_core_profile_register_entity');
  // Vertical tabs.
  foreach ($entity_handlers as $entity_handler_name) {
    $entity_handler = new $entity_handler_name();
    $entity_info = $entity_handler->entityInfo();

    $entity_type = $entity_info['entity'];
    $entity_label = $entity_info['entity_label'];
    $entity_config = isset($profile->entities[$entity_type]) ? $profile->entities[$entity_type] : array();

    $form['entities'][$entity_type] = array(
      '#type' => 'fieldset',
      '#title' => $entity_label,
    );

    $form['entities'][$entity_type]['label'] = array(
      '#type' => 'value',
      '#value' => $entity_label,
    );

    $form['entities'][$entity_type]['status'] = array(
      '#type' => 'checkbox',
      '#title' => t('Add a @entity', array('@entity' => $entity_label)),
      '#description' => t('Check this box to create an entity of this type when the profile form is submitted.'),
      '#default_value' => empty($entity_config['status']) ? FALSE : $entity_config['status'],
    );

    $entity_info = entity_get_info($entity_type);
    $bundle_options = array();
    foreach ($entity_info['bundles'] as $bundle => $bundle_info) {
      $bundle_options[$bundle] = $bundle_info['label'];
    }
    $form['entities'][$entity_type]['bundle'] = array(
      '#type' => 'select',
      '#title' => t('@entity Type', array('@entity' => $entity_label)),
      '#options' => $bundle_options,
      '#empty_option' => t('--Select--'),
      '#empty_value' => '',
      '#description' => t('Select type of the entity to create.'),
      '#states' => array(
        'visible' => array(
          ':input[name="entities[' . $entity_type . '][status]"]' => array('checked' => TRUE),
        ),
      ),
      '#ajax' => array(
        'callback' => '_crm_core_profile_entity_fields',
        'wrapper' => 'crm-core-profile-entity-' . strtr($entity_type, '_', '-') . '-fields-container',
      ),
      '#default_value' => empty($entity_config['bundle']) ? '' : $entity_config['bundle'],
    );

    $form['entities'][$entity_type]['fields_container'] = array(
      '#type' => 'container',
      '#prefix' => '<div id="' . 'crm-core-profile-entity-' . strtr($entity_type, '_', '-') . '-fields-container' . '">',
      '#suffix' => '</div>',
      '#states' => array(
        'visible' => array(
          ':input[name="entities[' . $entity_type . '][status]"]' => array('checked' => TRUE),
        ),
      ),
    );

    if (isset($entity_config['bundle'])) {
      $entity_fields = $entity_handler->entityProperties();
      foreach (field_info_instances($entity_type, $entity_config['bundle']) as $field) {
        // Allow entity handlers to override labels
        if (isset($entity_fields[$field['field_name']])) {
          if (empty($entity_fields[$field['field_name']])) {
            // Remove properties with empty label
            unset($entity_fields[$field['field_name']]);
          }
          continue;
        }
        $entity_fields[$field['field_name']] = check_plain($field['label']);
      }
      unset($field);
      $form['entities'][$entity_type]['fields_container']['fields'] = array(
        '#type' => 'select',
        '#title' => t('@entity Fields', array('@entity' => $entity_label)),
        '#multiple' => TRUE,
        '#options' => $entity_fields,
        '#empty_option' => t('--Select--'),
        '#empty_value' => '',
        '#description' => t('Select fields of the entity to display.'),
        '#states' => array(
          'invisible' => array(
            ':input[name="entities[' . $entity_type . '][bundle]"]' => array('value' => ''),
          ),
        ),
        '#ajax' => array(
          'callback' => '_crm_core_profile_fields',
          'wrapper' => 'fields-container',
        ),
        '#default_value' => empty($entity_config['fields_container']['fields']) ? '' : $entity_config['fields_container']['fields'],
      );
    }

    if (module_exists('token')) {
      // We must load token values here to show them on the options form.
      drupal_add_js(drupal_get_path('module', 'token') . '/token.js');
      drupal_add_css(drupal_get_path('module', 'token') . '/token.css');
      drupal_add_library('token', 'treeTable');
    }

    $form['entities'][$entity_type]['token'] = array(
      '#type' => 'fieldset',
      '#title' => t('Token replacement'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#states' => array(
        'visible' => array(
          ':input[name="entities[' . $entity_type . '][status]"]' => array('checked' => TRUE),
        ),
      ),
    );

    if (module_exists('token')) {
      $form['entities'][$entity_type]['token']['tokens'] = array(
        '#theme' => 'token_tree',
        '#token_types' => array(token_get_entity_mapping('entity', $entity_type)),
        '#dialog' => FALSE,
        // @todo: Maybe enable token browser to not have this load so long.
      );
    }
  }

  // Fields table.
  $form['fields'] = array(
    '#type' => 'container',
    '#tree' => TRUE,
    '#prefix' => '<div id="fields-container">',
    '#suffix' => '</div>',
  );

  if (!empty($profile->entities)) {
    foreach ($profile->entities as $entity_type => $entity_config) {
      if (is_array($entity_config) && isset($entity_handlers[$entity_type])) {
        $entity_handler_name = $entity_handlers[$entity_type];
        if (class_exists($entity_handler_name)) {
          $entity_handler = new $entity_handler_name();
          $entity_properties = $entity_handler->entityProperties();
          if (isset($entity_config['fields_container'])) {
            foreach ($entity_config['fields_container']['fields'] as $field_name) {
              if (array_key_exists($field_name, $entity_properties)) {
                $entity_handler->renderPropertyConfig($form, $entity_type, $entity_config['label'], $entity_config['bundle'], $field_name);
              }
              else {
                $field_info = field_info_field($field_name);
                $field_handler = $field_info['type'] . 'ProfileFieldHandler';
                if (class_exists($field_handler)) {
                  $field_handler_obj = new $field_handler();
                  $field_handler_obj->renderFieldConfig($form, $entity_type, $entity_config['label'], $entity_config['bundle'], $field_name);
                }
                else {
                  $field_handler_obj = new DefaultProfileFieldHandler();
                  $field_handler_obj->renderFieldConfig($form, $entity_type, $entity_config['label'], $entity_config['bundle'], $field_name);
                }
              }
            }
          }
        }
      }
    }
  }

  $form['#attached']['js'] = array(
    drupal_get_path('module', 'crm_core_profile') . '/js/profile_edit.js',
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => empty($profile) ? t('Create') : t('Save'),
  );
  $form['actions']['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
    '#weight' => 15,
    '#submit' => array('crm_core_profile_new_form_cancel'),
  );
  if (!empty($profile)) {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#weight' => 16,
      '#submit' => array('crm_core_profile_delete_submit'),
    );
  }

  return $form;
}

/**
 * New form helper.
 */
function _crm_core_profile_new_form_fields(&$form, $entity_label, $field_config, $sub_field = FALSE) {

  $field_separator = _crm_core_profile_field_separator($field_config);

  $form['fields'][$field_separator] = array();
  $row =& $form['fields'][$field_separator];

  $weight = isset($field_config['weight']) ? $field_config['weight'] : _crm_core_profile_default_weight();

  $row['#weight'] = isset($field_config['weight']) ? _crm_core_profile_weight_fix($field_config['weight']) : $weight;

  if ($sub_field) {
    $row['#attributes']['class'] = array('sub-field');
  }

  $row['field_name'] = array(
    '#type' => 'value',
    '#value' => $field_config['field_name'],
  );
  $row['field_item'] = array(
    '#type' => 'value',
    '#value' => $field_config['field_item'],
  );
  $row['handler'] = array(
    '#type' => 'value',
    '#value' => $field_config['handler'],
  );
  $row['entity_type'] = array(
    '#type' => 'value',
    '#value' => $field_config['entity_type'],
  );
  $row['bundle'] = array(
    '#type' => 'value',
    '#value' => $field_config['bundle'],
  );

  $row['status'] = array(
    '#type' => 'checkbox',
    '#default_value' => isset($field_config['status']) ? $field_config['status'] : FALSE,
  );

  $row['title_wrapper'] = array(
    '#type' => 'container',
  );
  $row['title_wrapper']['title'] = array(
    '#markup' => t('@field_label (%field_entity)', array(
        '@field_label' => $field_config['field_label'],
        '%field_entity' => $entity_label,
      )
    ),
  );

  $row['hidden'] = array(
    '#type' => 'checkbox',
    '#title' => t('Hidden'),
    '#title_display' => 'after',
    '#default_value' => isset($field_config['hidden']) ? $field_config['hidden'] : FALSE,
  );

  $field = field_info_field($field_config['field_name']);
  switch ($field['type']) {
    case 'list_integer':
    case 'list_float':
    case 'list_text':

      if (isset($field['settings']['allowed_values'])) {
        $options = $field['settings']['allowed_values'];
        array_unshift($options, t('--Please Select--'));
      }
      else {
        $options = array('' => t('Nothing to choose'));
      }

      $row['default'] = array(
        '#type' => 'select',
        '#title' => t('Default Value'),
        '#default_value' => isset($field_config['default']) ? $field_config['default'] : '',
        '#options' => $options,
      );
      break;

    default:
      $row['default'] = array(
        '#type' => 'textfield',
        '#title' => t('Default Value'),
        '#default_value' => isset($field_config['default']) ? $field_config['default'] : '',
        '#size' => 60,
        '#maxlength' => 200,
      );
  }

  $row['weight'] = array(
    '#type' => 'weight',
    '#title' => t('Weight for @title', array('@title' => $field_config['field_label'])),
    '#title_display' => 'invisible',
    '#default_value' => isset($field_config['weight']) ? $field_config['weight'] : $weight,
    '#delta' => DEFAULT_WEIGHT_DELTA,
  );
}

/**
 * Helper to fix display weight.
 */
function _crm_core_profile_weight_fix($weight) {
  if ($weight == 0) {
    // Table row positioned incorrectly if "#weight" is 0.
    $weight = 0.001;
  }

  return $weight;
}

/**
 * Ajax callback.
 */
function _crm_core_profile_entity_fields($form, $form_state) {
  $entity_type = $form_state['triggering_element']['#array_parents'][1];

  return $form['entities'][$entity_type]['fields_container'];
}

/**
 * Ajax callback.
 */
function _crm_core_profile_fields($form, $form_state) {
  return '<div id="fields-container">' . _theme_crm_core_profile_new_form_helper($form) . '</div>';
}

/**
 * Profile name validation callback.
 *
 * @param string $value
 *   Profile machine name.
 *
 * @return bool
 *   FALSE if unique.
 */
function crm_core_profile_new_form_machine_name_check($value) {

  $result = db_select('crm_core_profile')
    ->fields('crm_core_profile', array('name'))
    ->condition('name', $value)
    ->execute()
    ->fetchField();

  return $result;
}

/**
 * Validation callback.
 */
function crm_core_profile_new_form_validate($form, &$form_state) {
  $profile_config = $form_state['values'];
  unset($profile_config['entities']['entities__active_tab']);
  foreach ($profile_config['entities'] as $entity_type => $entity_config) {
    if ($entity_config['status'] && empty($entity_config['bundle'])) {
      form_error($form['entities'][$entity_type]['bundle'], t('You must select type of %entity to create.',
        array(
          '%entity' => $entity_config['label'],
        )
      ));
    }
  }
}

/**
 * Submission callback.
 */
function crm_core_profile_new_form_submit($form, &$form_state) {

  $values = $form_state['values'];
  unset($values['submit'], $values['form_build_id'], $values['form_token'], $values['form_id'], $values['op']);
  unset($values['entities']['entities__active_tab']);

  if (empty($values['fields'])) {
    $values['fields'] = array();
  }

  if (isset($form['#profile']->clone)) {
    // Iterate through profile props,
    foreach ($form['#profile'] as $k => $v) {
      // Fill in only props that didn't change or not set.
      if (!isset($values[$k])) {
        $values[$k] = $v;
      }
    }
  }

  // Adding entities settings from configuration page.
  if (isset($form['#profile'])) {
    foreach ($values['entities'] as $entity_type => $entity_config) {
      if (isset($form['#profile']->entities[$entity_type])) {
        $values['entities'][$entity_type] += $form['#profile']->entities[$entity_type];
      }
    }
  }

  crm_core_profile_save((object) $values);

  if (empty($form['#profile']) || isset($form['#profile']->clone)) {
    $form_state['redirect'] = 'admin/structure/crm-core/profile/manage/' . $values['name'] . '/settings';
  }
  else {
    $form_state['redirect'] = 'admin/structure/crm-core/profile';
  }
  drupal_set_message('Your profile has been successfully saved.');
}

/**
 * Submit handler for "Cancel" button.
 */
function crm_core_profile_new_form_cancel() {
  drupal_goto('admin/structure/crm-core/profile');
}

/**
 * New profile theme implementation.
 */
function theme_crm_core_profile_new_form($variables) {
  $form = $variables['form'];

  if (isset($form['fields'])) {
    drupal_add_tabledrag('fields-table', 'order', 'sibling', 'field-weight');
    $form['fields']['content']['#markup'] = _theme_crm_core_profile_new_form_helper($form);
  }

  return drupal_render_children($form);
}

/**
 * Theme helper.
 */
function _theme_crm_core_profile_new_form_helper(&$form) {

  $rows = array();

  foreach (element_children($form['fields'], TRUE) as $id) {
    $field =& $form['fields'][$id];
    $field['weight']['#attributes']['class'][] = 'field-weight';
    $indentation_size = 0;
    if (isset($field['#attributes']['class'])) {
      $indentation_size = in_array('sub-field', $field['#attributes']['class']) ? 1 : 0;
    }

    $row = array(
      'data' => array(),
      'class' => array('draggable'),
    );

    $row['data'][] = theme('indentation', array('size' => $indentation_size)) . drupal_render($field['status']);
    $row['data'][] = drupal_render($field['title_wrapper']);
    if (isset($field['description_wrapper'])) {
      $row['data'][] = drupal_render($field['weight']);
      $row['data'][] = array(
        'data' => drupal_render($field['description_wrapper']),
        'colspan' => '2',
      );
    }
    else {
      $row['data'][] = drupal_render($field['weight']);
      $row['data'][] = drupal_render($field['hidden']);
      $row['data'][] = drupal_render($field['default']);
    }

    $rows[] = $row;
  }

  return theme('table', array(
    'rows' => $rows,
    'attributes' => array('id' => 'fields-table'),
  ));
}

/**
 * Presents the form for profile deletion.
 */
function crm_core_profile_delete_form($form, &$form_state, $crm_core_profile) {
  $form['name'] = array(
    '#type' => 'value',
    '#value' => $crm_core_profile->name,
  );

  $question = t('Are you sure you want to delete the profile @profile', array('@profile' => $crm_core_profile->name));
  $path = 'admin/structure/crm-core/profile';

  return confirm_form($form, $question, $path);
}

/**
 * Submit handler.
 */
function crm_core_profile_delete_form_submit($form, &$form_state) {
  $machine_name = $form_state['values']['name'];
  crm_core_profile_delete($machine_name);

  $form_state['redirect'] = 'admin/structure/crm-core/profile';
  drupal_set_message(t('The profile form has been successfully deleted'));
  menu_rebuild();
}

/**
 * Profile settings form.
 */
function crm_core_profile_settings_form($form, &$form_state, $profile) {

  drupal_set_title(t('Profile settings for @profile', array('@profile' => $profile->label)));
  $form_state['profile'] = $profile;

  $form['display'] = array(
    '#type' => 'fieldset',
    '#title' => t('Display Settings'),
    '#collapsible' => TRUE,
  );

  $form['display']['page'] = array(
    '#type' => 'checkbox',
    '#title' => t('Create a page'),
    '#default_value' => $profile->page,
  );

  $form['display']['page_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Page Title'),
    '#default_value' => $profile->page_title,
    '#description' => 'Please provide a title for the page.',
    '#states' => array(
      'invisible' => array(
        ':input[name="page"]' => array('checked' => FALSE),
      ),
    ),
  );

  $form['display']['page_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Page Path'),
    '#default_value' => $profile->page_path,
    '#description' => 'Please provide a path to the page. Paths can include %user and %crm_core_contact as tokens. If you include a token, you must also specify prepopulation settings below.',
    '#states' => array(
      'invisible' => array(
        ':input[name="page"]' => array('checked' => FALSE),
      ),
    ),
  );

  $form['display']['page_redirect'] = array(
    '#type' => 'textfield',
    '#title' => t('Redirect Path'),
    '#default_value' => $profile->page_redirect,
    '#description' => 'Please provide a valid path where users will be taken after filling out the profile. Leave this field blank to return to the same page.',
    '#states' => array(
      'invisible' => array(
        ':input[name="page"]' => array('checked' => FALSE),
      ),
    ),
  );

  $form['display']['block'] = array(
    '#type' => 'checkbox',
    '#title' => t('Create a block'),
    '#default_value' => $profile->block,
  );

  $form['display']['block_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Block Title'),
    '#default_value' => $profile->block_title,
    '#states' => array(
      'invisible' => array(
        ':input[name="block"]' => array('checked' => FALSE),
      ),
    ),
  );

  $message_description = 'Please enter a message to display to users each time this form is successfully submitted.'
    . ' Maximum length is 512 characters.';
  $form['display']['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message to users'),
    '#cols' => 60,
    '#rows' => 3,
    '#default_value' => $profile->message,
    '#description' => t($message_description),
  );

  $form['roles_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Permissions'),
    '#collapsible' => TRUE,
  );

  $form['roles_wrapper']['description_wrapper'] = array(
    '#type' => 'container',
  );
  $permissions_description = 'Please select the user roles who will have access to this profile. Users who do not'
    . ' belong to one or more of these groups will be unable to view the profile. No restrictions by default.';
  $form['roles_wrapper']['description_wrapper']['description'] = array(
    '#markup' => t($permissions_description),
  );

  // selects roles associated with the profile
  $form['roles_wrapper']['roles'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Select roles'),
    '#options' => drupal_map_assoc(user_roles()),
    '#default_value' => isset($profile->roles) ? drupal_map_assoc($profile->roles) : array(),
  );

  /**
   * Entity handlers are used to add fields for settings to the form based on the
   * entities that are selected.
   */
  $entity_handlers = module_invoke_all('crm_core_profile_register_entity');

  foreach ($entity_handlers as $entity_handler_name) {
    $entity_handler = new $entity_handler_name();
    $entity_handler->configEntityForm($profile, $form);
  }

  // submit buttons
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['actions']['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
    '#weight' => 15,
    '#submit' => array('crm_core_profile_new_form_cancel'),
  );
  $form['actions']['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
    '#weight' => 16,
    '#submit' => array('crm_core_profile_delete_submit'),
  );

  return $form;
}

/**
 * Settings form validator.
 *
 * This handler validates submissions from the settings form. The validation
 * functions are executed dynamically based on what entities are being handled
 * in the profile.
 */
function crm_core_profile_settings_form_validate($form, &$form_state) {

  $profile = $form_state['profile'];
  $values = $form_state['values'];

  // handling page creation. checking to see if configured paths are available.
  if ($values['page'] && !empty($values['page_path'])) {
    // Checking if this path is already taken.
    if ($values['page_path'] != $profile->page_path) {
      $menu_item = menu_get_item($values['page_path']);
      if ($menu_item) {
        form_set_error('page_path', t('A page with that path already exists. Please select another path for this page.'));
      }
    }
  }

  // validate fields created by entity handlers
  $entity_handlers = module_invoke_all('crm_core_profile_register_entity');
  foreach ($entity_handlers as $entity_handler_name) {
    $entity_handler = new $entity_handler_name();
    $entity_handler->configEntityFormValidate($form, $form_state);
  }
}

/**
 * Settings form submit handler.
 *
 * This handler saves some basic settings for the profile on it's own,
 * and executes the
 *
 */
function crm_core_profile_settings_form_submit($form, &$form_state) {
  $profile = $form_state['profile'];
  $profile_settings = $form_state['values'];

  // make sure we are only saving page paths when we are creating a page for the profile
  if (!$profile_settings['page']) {
    unset($profile_settings['page_path']);
    unset($profile_settings['page_title']);
  }

  // 
  $entity_handlers = module_invoke_all('crm_core_profile_register_entity');
  foreach ($entity_handlers as $entity_handler_name) {
    $entity_handler = new $entity_handler_name();
    $entity_info = $entity_handler->entityInfo();
    $entity_type = $entity_info['entity'];
    if (isset($profile_settings[$entity_type])) {
      foreach ($profile_settings[$entity_type] as $key => $value) {
        $profile->entities[$entity_type][$key] = $value;
      }
      unset($profile_settings[$entity_type]);
    }
  }

  foreach ($profile_settings as $key => $value) {
    $profile->{$key} = $value;
  }
  $profile->roles = array();
  foreach ($profile_settings['roles'] as $role_name => $role_status) {
    if ($role_name === $role_status) {
      $profile->roles[] = $role_name;
    }
  }

  crm_core_profile_save($profile);
  // Rebuilding menu to register profile path.
  if ($profile_settings['page']) {
    menu_rebuild();
  }

  $form_state['redirect'] = 'admin/structure/crm-core/profile';
  drupal_set_message('Your profile settings have been saved');
}

/**
 * Submit handler for "Delete" button.
 */
function crm_core_profile_delete_submit($form, &$form_state) {
  $profile = isset($form['#profile']) ? $form['#profile'] : $form_state['profile'];
  drupal_goto('admin/structure/crm-core/profile/manage/' . $profile->name . '/delete');
}

/**
 * Profile export form.
 */
function crm_core_profile_export_form($form, &$form_state, $profile) {

  drupal_set_title(t('Export @profile', array('@profile' => $profile->label)));
  module_load_include('inc', 'ctools', 'includes/export');

  $form['profile'] = array(
    '#type' => 'textarea',
    '#rows' => 50,
    '#default_value' => ctools_export_crud_export('crm_core_profile', $profile),
  );

  return $form;
}

/**
 * Profile import form.
 */
function crm_core_profile_import_form($form, &$form_state) {

  $form['label'] = array(
    '#type' => 'textfield',
    '#title' => t('Profile name'),
    '#description' => t('Please enter the profile form name.'),
    '#maxlength' => 28,
    '#default_value' => empty($form_state['values']) ? '' : $form_state['values']['label'],
  );

  $name_description = 'Enter the name to use for this profile if it is different from the source profile. Leave blank'
    . ' to use the name of the source profile.';
  $form['name'] = array(
    '#type' => 'textfield',
    '#maxlength' => 28,
    '#title' => t('Profile machine readable name'),
    '#default_value' => empty($form_state['values']) ? '' : $form_state['values']['name'],
    '#description' => t($name_description),
  );

  $form['name_override'] = array(
    '#type' => 'checkbox',
    '#title' => t('Replace an existing profile if one exists with the same name'),
    '#default_value' => empty($form_state['values']) ? FALSE : $form_state['values']['name_override'],
  );

  $form['profile'] = array(
    '#type' => 'textarea',
    '#title' => t('Paste CRM Core Profile code here'),
    '#default_value' => empty($form_state['values']) ? '' : $form_state['values']['profile'],
    '#states' => array(
      'required' => array(
        ':input[name="profile"]' => array('empty' => TRUE),
      ),
    ),
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['import'] = array(
    '#type' => 'submit',
    '#value' => t('Import'),
  );
  $form['actions']['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
    '#weight' => 15,
    '#submit' => array('crm_core_profile_new_form_cancel'),
  );

  return $form;
}

/**
 * Profile import form validate handler.
 */
function crm_core_profile_import_form_validate($form, &$form_state) {
  $values = $form_state['values'];
  if (!empty($values['profile'])) {
    // Declaring $crm_core_profile just to fool syntax highlighter.
    $crm_core_profile = array();
    ob_start();
    eval($values['profile']);
    ob_end_clean();

    if (!is_object($crm_core_profile)) {
      form_set_error('profile', t('Unable to interpret profile code.'));
    }

    // Profile name must be alphanumeric or underscores, no other punctuation.
    if (!empty($values['name']) && preg_match('/[^a-zA-Z0-9_]/', $values['name'])) {
      form_set_error('name', t('Profile name must be alphanumeric or underscores only.'));
    }

    $machine_name_check = FALSE;
    // User didn't provided new machine name, checking from code.
    if (empty($values['name']) && crm_core_profile_new_form_machine_name_check($crm_core_profile->name)) {
      // Profile already exists.
      $machine_name_check = TRUE;
    }
    elseif (crm_core_profile_new_form_machine_name_check($values['name'])) {
      $machine_name_check = TRUE;
    }
    if (!$values['name_override'] && $machine_name_check) {
      form_set_error('name', t('A profile by that name already exists, please choose a different name.'));
    }
    $form_state['profile'] = $crm_core_profile;
  }
  else {
    form_set_error('profile', t('You must put exported CRM Core Profile code into that field.'));
  }
}

/**
 * Profile import form submit handler.
 */
function crm_core_profile_import_form_submit($form, &$form_state) {
  $profile = $form_state['profile'];
  if (!empty($form_state['values']['label'])) {
    $profile->label = $form_state['values']['label'];
  }
  if (!empty($form_state['values']['name'])) {
    $profile->name = $form_state['values']['name'];
  }
  crm_core_profile_save($profile);
  drupal_set_message(t('CRM Core Profile %profile was successfully imported.', array('%profile' => $profile->label)));
  $edit_path = 'admin/structure/crm-core/profile/manage/' . $profile->name . '/edit';
  $form_state['redirect'] = $edit_path;
}
