<?php

/**
 * @file 
 * Describes hooks used in CRM Core Profile
 */

/**
 * Fires just before a contact is saved through a CRM Core Profile.
 * 
 * This hook is necessary because of the way CRMCoreContactEntity->match works.
 * When checking for duplicates, CRM Core provides the ability to modify a contact record
 * during the matching process. CRM Core Profile will update any contact record values
 * that are passed into the form, but it will not necessarily carry over any changes to
 * contact records that have been made via CRMCoreContactEntity->match.
 * 
 * This scenario will not affect profiles that are prepopulated, or profiles that do not 
 * use contact matching.
 * 
 * Of course, this hook can be used for other reasons as well. Use it any time you need to 
 * modify a value being stored as part of a contact record before it is saved. For instance, 
 * if you need a store the source of a contact, this might be a good way to do it.
 * 
 * This hook takes an associative array of values, which includes the contact record passed
 * by reference. Any modifications to the contact record will be saved when the contact is 
 * saved through the profile. 
 * 
 * @param array $values
 * 	An associative array, including a key for the contact record passed by reference.
 */
function hook_crm_core_profile_contact_presave(&$values){

  $contact = $values['contact'];
  
  // when modifying a contact record, you could store the modifications in a static variable...
  $my_custom_changes = & drupal_static(__FUNCTION__); 
    
  // ... and check to make sure they are staged to be committed. This would eliminate any issues
  // with modifications that are being made to existing contacts.
  if(isset($my_custom_changes[$values['contact']->contact_id]) && !isset($values['contact']->my_custom_changes)){
    $values['contact']->my_custom_changes = $my_custom_changes[$values['contact']->contact_id];
  }
  
}






/**
 * Older hooks - may or may not be used as we finish up the rewrite
 */

/**
 *
 * Notify Drupal when a CRM Core Profile entry form is being submitted/saved
 *
 * @param array $crm_core_profile
 *   Information of the CRM Core profile that's associated with the CRM Core profile entry form
 *   Being submitted
 * @param array $form_state_values
 *   An array of form_state['values'] data submitted by the CRM Core profile entry form
 *
 */
function hook_crm_core_profile_save($crm_core_profile, $form_state_values) {
  
  // trigger an email to the user that submitted the form;
  global $user;
  
  drupal_mail('crm_core_profile_notification', 'crm_core_profile_notification', $user->mail, LANGUAGE_NONE, array(), variable_get('site_mail', NULL), TRUE);      
}


/**
 *
 * Notify Drupal when a CRM Core Profile entry form is submitted and the contact is saved
 *
 * @param array $crm_core_profile
 *   Information of the CRM Core profile that's associated with the CRM Core profile entry form
 *   Being submitted
 * @param stdClass() $contact
 *   An object that contains the CRM Core Contact data that's created/saved/updated
 *
 */
function hook_crm_core_profile_save_contact($crm_core_profile, $contact) {
  
  // saving additional information into the contact object
  $contact->field_contact_source = 'google_cpc';
  crm_core_contact_save($contact);
  
}

/**
 *
 * Offers the ability to change form elements to be rendered on the CRM Core Profile Entry form
 *
 * @param array $form
 *   The form tree
 *
 * @param array $render_fields
 *   An array of fields to be rendered, it is a sorting array with
 *   keyed by weight and value
 *
 */
function hook_crm_core_profile_theme_alter(&$form, &$render_fields) {
  $profile_activity = crm_core_profile_activity_load($form['profile_name']['#value']);
  
  if (empty($profile_activity)) {
    return; 
  }
  
  $fields = unserialize($profile_activity['fields']);
  foreach ($fields['weight'] as $field => $weight) {
      $render_fields[$weight['weight']['weight']] = "activity||" . $field;
  } 
}


/**
 *
 * Notify Drupal when a CRM Core Profile is being deleted
 *
 * @param string $machine_name
 *   The name of the CRM Core Profile being deleted
 *
 */
function hook_crm_core_profile_delete($machine_name) {
    db_delete('crm_core_profile_activity')
    ->condition('name', $machine_name)
    ->execute();  
}

/**
 * Update administrative form with ajax on contact bundle type element update
 *
 * @return ajax_commands array.
 */
function hook_crm_core_profile_new_form_bundle_type_ajax_callback() {
  $html = drupal_render($form['relation']['bundle_type']);
  $commands[] = ajax_command_replace(
    '#crm-core-profile-relation-bundle-type',
    $html
  );
  return $commands;
}
